@@ -51,7 +51,7 @@
 		if (!is.null(processors) && processors < 1)
 			stop("processors must be at least 1.")
 		if (is.null(processors)) {
-			processors <- .detectCores()
+			processors <- .Call("detectCores", PACKAGE="DECIPHER")
 		} else {
 			processors <- as.integer(processors)
 		}
@@ -236,7 +236,7 @@ DesignSignatures <- function(dbFile,
 	if (!is.null(processors) && processors < 1)
 		stop("processors must be at least 1.")
 	if (is.null(processors)) {
-		processors <- .detectCores()
+		processors <- .Call("detectCores", PACKAGE="DECIPHER")
 	} else {
 		processors <- as.integer(processors)
 	}
@@ -1528,7 +1528,7 @@ DesignSignatures <- function(dbFile,
 		d <- dist(signatures,
 			method="maximum")
 		if (length(d) > 0) {
-			c <- TreeLine(myDistMatrix=d,
+			c <- Treeline(myDistMatrix=d,
 				type="clusters",
 				method="single",
 				cutoff=levels/5,
@@ -1993,7 +1993,7 @@ DesignSignatures <- function(dbFile,
 				d <- dist(signatures,
 					method="maximum")
 				if (length(d) > 0) {
-					c <- TreeLine(myDistMatrix=d,
+					c <- Treeline(myDistMatrix=d,
 						type="clusters",
 						method="single",
 						cutoff=levels/5,
