@@ -1,7 +1,8 @@
 ReadDendrogram <- function(file,
 	convertBlanks=TRUE,
 	internalLabels=TRUE,
-	keepRoot=TRUE) {
+	keepRoot=TRUE,
+	quote="'") {
 	
 	# error checking
 	if (!is.logical(convertBlanks))
@@ -10,8 +11,18 @@ ReadDendrogram <- function(file,
 		stop("internalLabels must be a logical.")
 	if (!is.logical(keepRoot))
 		stop("keepRoot must be a logical.")
+	quote <- match(quote, c("'", '"'))
+	if (is.na(quote))
+		stop("Invalid quote.")
+	
+	if (is(file, "file") || file.exists(file)) {
+		r <- readLines(file, warn=FALSE)
+	} else if (is.character(file)) {
+		r <- file
+	} else {
+		stop("file must be a file connection, file path, or the contents of a Newick formatted file.")
+	}
 	
-	r <- readLines(file, warn=FALSE)
 	w <- which(nchar(r) > 0)
 	if (length(w) > 1) {
 		r <- paste(r[w], collapse="")
@@ -22,7 +33,9 @@ ReadDendrogram <- function(file,
 	}
 	
 	r <- strsplit(r,
-		'(?=[\\[\\](),:;])(?=([^"]*"[^"]*")*[^"]*$)',
+		ifelse(quote == 1L,
+			"(?=[\\[\\](),:;])(?=([^']*'[^']*')*[^']*$)",
+			'(?=[\\[\\](),:;])(?=([^"]*"[^"]*")*[^"]*$)'),
 		perl=TRUE)[[1]]
 	r <- gsub("^\\s+|\\s+$", "", r)
 	w <- which(r == "")
@@ -31,7 +44,11 @@ ReadDendrogram <- function(file,
 	
 	getLab <- function(LAB) {
 		# convert underscores to spaces in unquoted labels
-		lab <- gsub("^\"(.*)\"$", "\\1", LAB)
+		lab <- gsub(ifelse(quote == 1L,
+				"^\'(.*)\'$",
+				"^\"(.*)\"$"),
+			"\\1",
+			LAB)
 		if (convertBlanks && nchar(lab) == nchar(LAB))
 			lab <- gsub("_", " ", lab, fixed=TRUE)
 		if (nchar(lab) != nchar(LAB))
@@ -77,7 +94,7 @@ ReadDendrogram <- function(file,
 				} else if (i < length(r) && r[i + 1L] == ";") {
 					i <<- i + 1L
 				} else if (i <= length(r) && r[i] != ";") {
-					stop("Unsupported file formatting.")
+					stop("Unsupported formatting.")
 				}
 				break
 			} else if (r[i] == "(") {
@@ -109,7 +126,7 @@ ReadDendrogram <- function(file,
 			} else if (r[i] == " ") {
 				i <<- i + 1L
 			} else {
-				stop("Unsupported file formatting.")
+				stop("Unsupported formatting.")
 			}
 		}
 		
