@@ -6,6 +6,7 @@
 // for OpenMP parallel processing
 #ifdef _OPENMP
 #include <omp.h>
+#undef match
 #endif
 
 /*
@@ -127,7 +128,7 @@ SEXP indexByContig(SEXP starts, SEXP ends, SEXP order, SEXP index, SEXP widths)
 	return ret_list;
 }
 
-SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, SEXP y_i, SEXP y_f, SEXP weights, SEXP sepCost, SEXP sepPower, SEXP gapCost, SEXP gapPower, SEXP shiftCost, SEXP codingCost, SEXP maxSep, SEXP maxGap, SEXP ordering, SEXP minScore, SEXP maxW, SEXP allowOverlap)
+SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, SEXP y_i, SEXP y_f, SEXP weights, SEXP sepCost, SEXP sepPower, SEXP gapCost, SEXP gapPower, SEXP shiftCost, SEXP codingCost, SEXP maxSep, SEXP maxGap, SEXP ordering, SEXP minScore, SEXP maxW)
 {
 	int i, k, max, dy, dx, sep, gap, xok, xoj;
 	int *xs = INTEGER(x_s);
@@ -150,12 +151,11 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 	double totW = asReal(maxW);
 	int *xo = INTEGER(ordering);
 	double minS = asReal(minScore);
-	int aO = asInteger(allowOverlap);
 	
 	// initialize an array of sep multipliers
-	double *SEPS = Calloc(maxS + 1, double); // initialized to zero
+	double *SEPS = R_Calloc(maxS + 1, double); // initialized to zero
 	// initialize an array of gap multipliers
-	double *GAPS = Calloc(maxG + 1, double); // initialized to zero
+	double *GAPS = R_Calloc(maxG + 1, double); // initialized to zero
 	for (i = 1; i <= maxS; i++)
 		SEPS[i] = pow(i, sepP)*sepC;
 	for (i = 1; i <= maxG; i++)
@@ -170,13 +170,13 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 	double temp, score;
 	
 	// initialize an array of activities
-	int *A = Calloc(l, int); // initialized to zero
+	int *A = R_Calloc(l, int); // initialized to zero
 	// initialize an array of scores
-	double *S = Calloc(l, double); // initialized to zero
+	double *S = R_Calloc(l, double); // initialized to zero
 	// initialize an array of return indicies
-	int *R = Calloc(l, int); // initialized to zero
+	int *R = R_Calloc(l, int); // initialized to zero
 	// initialize an array of origin indices
-	int *O = Calloc(l, int); // initialized to zero
+	int *O = R_Calloc(l, int); // initialized to zero
 	
 	while (i < l || j < (l - 1)) {
 		if (i < l &&
@@ -203,11 +203,11 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 					continue;
 				
 				dy = ys[i] - ye[xok] - 1;
-				if (dy <= 0)
+				if (dy < 0)
 					continue;
 				
 				dx = xs[i] - xe[xok] - 1;
-				if (dx <= 0)
+				if (dx < 0)
 					continue;
 				
 				if (dx > dy) {
@@ -319,16 +319,16 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 	int *p;
 	int n, min, overlap;
 	int size = 1000;
-	int **ptrs = Calloc(size, int *); // chains
-	int *lens = Calloc(size, int); // length of each chain
-	double *scores = Calloc(size, double); // score of each chain
+	int **ptrs = R_Calloc(size, int *); // chains
+	int *lens = R_Calloc(size, int); // length of each chain
+	double *scores = R_Calloc(size, double); // score of each chain
 	// start and end of the rectangle encompassing each chain
-	int *rectXS = Calloc(size, int);
-	int *rectXE = Calloc(size, int);
-	int *rectYS = Calloc(size, int);
-	int *rectYE = Calloc(size, int);
-	int *rectXI = Calloc(size, int);
-	int *rectYI = Calloc(size, int);
+	int *rectXS = R_Calloc(size, int);
+	int *rectXE = R_Calloc(size, int);
+	int *rectYS = R_Calloc(size, int);
+	int *rectYE = R_Calloc(size, int);
+	int *rectXI = R_Calloc(size, int);
+	int *rectYI = R_Calloc(size, int);
 	
 	for (i = 0; i < l; i++)
 		A[i] = 1; // re-activate all
@@ -385,44 +385,23 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 		}
 		
 		// pull back overlapping end
-		if (aO) { // allow overlaps in x or y
-			while (max != -1 &&
-				max >= O[max]) {
-				overlap = 0;
-				for (i = 0; i < count; i++) {
-					if (xi[max] == rectXI[i] &&
-						yi[max] == rectYI[i] &&
-						xe[max] >= rectXS[i] &&
-						xe[max] <= rectXE[i] &&
-						ye[max] >= rectYS[i] &&
-						ye[max] <= rectYE[i]) {
-						overlap = 1;
-						break;
-					}
-				}
-				if (overlap == 0)
+		while (max != -1 &&
+			max >= O[max]) {
+			overlap = 0;
+			for (i = 0; i < count; i++) {
+				if (xi[max] == rectXI[i] &&
+					yi[max] == rectYI[i] &&
+					xe[max] >= rectXS[i] &&
+					xe[max] <= rectXE[i] &&
+					ye[max] >= rectYS[i] &&
+					ye[max] <= rectYE[i]) {
+					overlap = 1;
 					break;
-				max = R[max];
-			}
-		} else {
-			while (max != -1 &&
-				max >= O[max]) {
-				overlap = 0;
-				for (i = 0; i < count; i++) {
-					if (xi[max] == rectXI[i] &&
-						yi[max] == rectYI[i] &&
-						((xe[max] >= rectXS[i] &&
-						xe[max] <= rectXE[i]) ||
-						(ye[max] >= rectYS[i] &&
-						ye[max] <= rectYE[i]))) {
-						overlap = 1;
-						break;
-					}
 				}
-				if (overlap == 0)
-					break;
-				max = R[max];
 			}
+			if (overlap == 0)
+				break;
+			max = R[max];
 		}
 		
 		if (max == -1) // no chain
@@ -432,46 +411,24 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 		min = max;
 		n = min;
 		j = 0;
-		if (aO) { // allow overlaps in x or y
-			while (n >= O[max]) {
-				overlap = 0;
-				for (i = 0; i < count; i++) {
-					if (xi[n] == rectXI[i] &&
-						yi[n] == rectYI[i] &&
-						xs[n] >= rectXS[i] &&
-						xs[n] <= rectXE[i] &&
-						ys[n] >= rectYS[i] &&
-						ys[n] <= rectYE[i]) {
-						overlap = 1;
-						break;
-					}
-				}
-				if (overlap == 1)
+		while (n >= O[max]) {
+			overlap = 0;
+			for (i = 0; i < count; i++) {
+				if (xi[n] == rectXI[i] &&
+					yi[n] == rectYI[i] &&
+					xs[n] >= rectXS[i] &&
+					xs[n] <= rectXE[i] &&
+					ys[n] >= rectYS[i] &&
+					ys[n] <= rectYE[i]) {
+					overlap = 1;
 					break;
-				j++;
-				min = n; // prior value of min
-				n = R[min];
-			}
-		} else {
-			while (n >= O[max]) {
-				overlap = 0;
-				for (i = 0; i < count; i++) {
-					if (xi[n] == rectXI[i] &&
-						yi[n] == rectYI[i] &&
-						((xs[n] >= rectXS[i] &&
-						xs[n] <= rectXE[i]) ||
-						(ys[n] >= rectYS[i] &&
-						ys[n] <= rectYE[i]))) {
-						overlap = 1;
-						break;
-					}
 				}
-				if (overlap == 1)
-					break;
-				j++;
-				min = n; // prior value of min
-				n = R[min];
 			}
+			if (overlap == 1)
+				break;
+			j++;
+			min = n; // prior value of min
+			n = R[min];
 		}
 		
 		if (j == 0)
@@ -511,13 +468,8 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 					}
 				}
 			}
-			if (aO) { // allow overlaps in x or y
-				if (minDx < 0 && minDy < 0) // completely within rectangle
-					continue;
-			} else {
-				if (minDx < 0 || minDy < 0) // completely overlapping in x or y
-					continue;
-			}
+			if (minDx < 0 && minDy < 0 && minX == minY) // completely within rectangle
+				continue;
 			
 			sep = 1e9;
 			gap = 1e9;
@@ -539,7 +491,7 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 			}
 			
 			if (merge) {
-				ptrs[minX] = Realloc(ptrs[minX], j + lens[minX], int);
+				ptrs[minX] = R_Realloc(ptrs[minX], j + lens[minX], int);
 				p = ptrs[minX];
 				
 				if (upX) { // new chain is last
@@ -582,18 +534,18 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 			
 			if (count >= size) {
 				size += 1000;
-				ptrs = Realloc(ptrs, size, int *);
-				lens = Realloc(lens, size, int);
-				scores = Realloc(scores, size, double);
-				rectXS = Realloc(rectXS, size, int);
-				rectXE = Realloc(rectXE, size, int);
-				rectYS = Realloc(rectYS, size, int);
-				rectYE = Realloc(rectYE, size, int);
-				rectXI = Realloc(rectXI, size, int);
-				rectYI = Realloc(rectYI, size, int);
+				ptrs = R_Realloc(ptrs, size, int *);
+				lens = R_Realloc(lens, size, int);
+				scores = R_Realloc(scores, size, double);
+				rectXS = R_Realloc(rectXS, size, int);
+				rectXE = R_Realloc(rectXE, size, int);
+				rectYS = R_Realloc(rectYS, size, int);
+				rectYE = R_Realloc(rectYE, size, int);
+				rectXI = R_Realloc(rectXI, size, int);
+				rectYI = R_Realloc(rectYI, size, int);
 			}
 			
-			ptrs[count] = Calloc(j, int);
+			ptrs[count] = R_Calloc(j, int);
 			lens[count] = j;
 			p = ptrs[count];
 			
@@ -617,18 +569,18 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 		}
 	}
 	
-	Free(SEPS);
-	Free(GAPS);
-	Free(A);
-	Free(S);
-	Free(R);
-	Free(O);
-	Free(rectXS);
-	Free(rectXE);
-	Free(rectYS);
-	Free(rectYE);
-	Free(rectXI);
-	Free(rectYI);
+	R_Free(SEPS);
+	R_Free(GAPS);
+	R_Free(A);
+	R_Free(S);
+	R_Free(R);
+	R_Free(O);
+	R_Free(rectXS);
+	R_Free(rectXE);
+	R_Free(rectYS);
+	R_Free(rectYE);
+	R_Free(rectXI);
+	R_Free(rectYI);
 	
 	SEXP ret, chains, chain, cs;
 	int *pchain;
@@ -644,7 +596,7 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 		for (j = 0; j < lens[i]; j++)
 			pchain[j] = p[j] + 1;
 		
-		Free(p);
+		R_Free(p);
 		
 		SET_VECTOR_ELT(chains, i, chain);
 		UNPROTECT(1);
@@ -652,9 +604,9 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 		pcs[i] = scores[i] - cost; // apply cost for first search
 	}
 	
-	Free(ptrs);
-	Free(lens);
-	Free(scores);
+	R_Free(ptrs);
+	R_Free(lens);
+	R_Free(scores);
 	
 	PROTECT(ret = allocVector(VECSXP, 2));
 	SET_VECTOR_ELT(ret, 0, chains);
@@ -667,7 +619,7 @@ SEXP chainSegments(SEXP x_s, SEXP x_e, SEXP x_i, SEXP x_f, SEXP y_s, SEXP y_e, S
 
 SEXP extendMatches(SEXP X1, SEXP X2, SEXP starts1, SEXP ends1, SEXP index1, SEXP starts2, SEXP ends2, SEXP index2, SEXP width1, SEXP width2, SEXP subMatrix, SEXP letters, SEXP dropScore, SEXP nThreads)
 {
-	int i, j, p1, p2, lkup1, lkup2, boundL1, boundR1, boundL2, boundR2;
+	int i, p1, p2, lkup1, lkup2, boundL1, boundR1, boundL2, boundR2;
 	
 	XStringSet_holder x1_set = hold_XStringSet(X1);
 	XStringSet_holder x2_set = hold_XStringSet(X2);
@@ -793,68 +745,147 @@ SEXP extendMatches(SEXP X1, SEXP X2, SEXP starts1, SEXP ends1, SEXP index1, SEXP
 			}
 		}
 	}
-	
-	SEXP basePackage, order;
-	PROTECT(basePackage = eval(lang2(install("getNamespace"), ScalarString(mkChar("base"))), R_GlobalEnv));
-	order = eval(lang3(install("sort.list"), ans0, R_NilValue), basePackage);
-	UNPROTECT(1);
-	int *o = INTEGER(order);
-	
-	char *pos1 = (char *) calloc(x1.length, sizeof(char)); // initialized to zero (thread-safe on Windows)
-	char *pos2 = (char *) calloc(x2.length, sizeof(char)); // initialized to zero (thread-safe on Windows)
-	for (j = l - 1; j >= 0; j--) {
-		i = o[j] - 1; // index
-		if (i1[i] == 1) {
-			boundL1 = 0;
-		} else {
-			boundL1 = w1[i1[i] - 2];
-		}
-		if (i2[i] == 1) {
-			boundL2 = 0;
-		} else {
-			boundL2 = w2[i2[i] - 2];
-		}
-		
-		// pull back overlapping start
-		p1 = rans1[i] + boundL1 - 1;
-		p2 = rans3[i] + boundL2 - 1;
-		while (p1 < rans2[i] + boundL1 - 1 && (pos1[p1] == 1 || pos2[p2] == 1)) {
-			lkup1 = lkup_row[(unsigned char)x1.ptr[p1]];
-			lkup2 = lkup_col[(unsigned char)x2.ptr[p2]];
-			if (lkup1 != NA_INTEGER && lkup2 != NA_INTEGER)
-				rans0[i] -= sM[lkup1 + lkup2];
-			p1++;
-			p2++;
-		}
-		rans1[i] = p1 + 1 - boundL1;
-		rans3[i] = p2 + 1 - boundL2;
-		
-		// pull back overlapping end
-		p1 = rans2[i] + boundL1 - 1;
-		p2 = rans4[i] + boundL2 - 1;
-		while (p1 >= rans1[i] + boundL1 && (pos1[p1] == 1 || pos2[p2] == 1)) {
-			lkup1 = lkup_row[(unsigned char)x1.ptr[p1]];
-			lkup2 = lkup_col[(unsigned char)x2.ptr[p2]];
-			if (lkup1 != NA_INTEGER && lkup2 != NA_INTEGER)
-				rans0[i] -= sM[lkup1 + lkup2];
-			p1--;
-			p2--;
-		}
-		rans2[i] = p1 + 1 - boundL1;
-		rans4[i] = p2 + 1 - boundL2;
-		
-		// mark positions as occupied
-		p1 = rans2[i] + boundL1;
-		p2 = rans4[i] + boundL2;
-		while (p1 >= rans1[i] + boundL1) {
-			p1--;
-			pos1[p1] = 1;
-			p2--;
-			pos2[p2] = 1;
-		}
-	}
 	free(lkup_row);
 	free(lkup_col);
+	
+	SEXP ret_list;
+	PROTECT(ret_list = allocVector(VECSXP, 5));
+	SET_VECTOR_ELT(ret_list, 0, ans0);
+	SET_VECTOR_ELT(ret_list, 1, ans1);
+	SET_VECTOR_ELT(ret_list, 2, ans2);
+	SET_VECTOR_ELT(ret_list, 3, ans3);
+	SET_VECTOR_ELT(ret_list, 4, ans4);
+	
+	UNPROTECT(6);
+	
+	return ret_list;
+}
+
+// pull back overlap (within buffer) until start equals end
+// assign a score proportional to number of positions kept
+// give duplicates zero score (and positions uninitialized)
+SEXP withdrawMatches(SEXP order, SEXP starts1, SEXP ends1, SEXP index1, SEXP starts2, SEXP ends2, SEXP index2, SEXP width1, SEXP width2, SEXP score, SEXP bufferSize)
+{
+	int i, j, k, p1, p2, o1, o2, len, maxLen, boundL1, boundL2, temp1, temp2;
+	
+	int *o = INTEGER(order);
+	int l = length(starts1);
+	int *s1 = INTEGER(starts1);
+	int *e1 = INTEGER(ends1);
+	int *i1 = INTEGER(index1);
+	int *s2 = INTEGER(starts2);
+	int *e2 = INTEGER(ends2);
+	int *i2 = INTEGER(index2);
+	int *w1 = INTEGER(width1);
+	int l1 = w1[length(width1) - 1];
+	int *w2 = INTEGER(width2);
+	int l2 = w2[length(width2) - 1];
+	double *s = REAL(score);
+	int buffer = asInteger(bufferSize);
+	int negBuffer = -buffer;
+	
+	SEXP ans0, ans1, ans2, ans3, ans4;
+	PROTECT(ans0 = allocVector(REALSXP, l));
+	double *rans0 = REAL(ans0);
+	PROTECT(ans1 = allocVector(INTSXP, l));
+	int *rans1 = INTEGER(ans1);
+	PROTECT(ans2 = allocVector(INTSXP, l));
+	int *rans2 = INTEGER(ans2);
+	PROTECT(ans3 = allocVector(INTSXP, l));
+	int *rans3 = INTEGER(ans3);
+	PROTECT(ans4 = allocVector(INTSXP, l));
+	int *rans4 = INTEGER(ans4);
+	
+	int *pos1 = (int *) malloc(l1*sizeof(int)); // thread-safe on Windows
+	int *pos2 = (int *) malloc(l2*sizeof(int)); // thread-safe on Windows
+	
+	for (i = 0; i < l1; i++)
+		pos1[i] = negBuffer;
+	for (i = 0; i < l2; i++)
+		pos2[i] = negBuffer;
+	
+	j = l - 1; // iterator
+	if (j >= 0) {
+		k = o[j] - 1;
+		rans0[k] = s[k];
+	}
+	while (j >= 0) {
+		i = k; // current index
+		j--;
+		if (j >= 0) {
+			k = o[j] - 1; // next index
+			if (s1[i] == s1[k] && s2[i] == s2[k] &&
+				e1[i] == e1[k] && e2[i] == e2[k] &&
+				i1[i] == i1[k] && i2[i] == i2[k]) { // duplicated
+				rans0[k] = 0; // next score is zero (leave bounds uninitialized)
+			} else {
+				rans0[k] = s[k];
+			}
+		}
+		
+		// itialize
+		rans1[i] = s1[i];
+		rans2[i] = e1[i];
+		rans3[i] = s2[i];
+		rans4[i] = e2[i];
+		
+		if (rans0[i] > 0) { // not a duplicate
+			if (i1[i] == 1) {
+				boundL1 = 0;
+			} else {
+				boundL1 = w1[i1[i] - 2];
+			}
+			if (i2[i] == 1) {
+				boundL2 = 0;
+			} else {
+				boundL2 = w2[i2[i] - 2];
+			}
+			
+			// pull back overlap
+			p1 = s1[i] + boundL1 - 1;
+			p2 = s2[i] + boundL2 - 1;
+			o1 = p1;
+			o2 = p2;
+			len = 0;
+			maxLen = 0;
+			while (p1 < e1[i] + boundL1 - 1) {
+				if ((p2 + 1 >= pos1[p1] - buffer && p2 + 1 <= pos1[p1] + buffer) ||
+					(p1 + 1 >= pos2[p2] - buffer && p1 + 1 <= pos2[p2] + buffer)) {
+					o1 = ++p1;
+					o2 = ++p2;
+					len = 0;
+				} else { // no overlap
+					len++;
+					p1++;
+					p2++;
+					if (len > maxLen) {
+						maxLen = len;
+						rans1[i] = o1 + 1 - boundL1;
+						rans2[i] = p1 - boundL1;
+						rans3[i] = o2 + 1 - boundL2;
+						rans4[i] = p2 - boundL2;
+					}
+				}
+			}
+			
+			// mark positions as occupied
+			p1 = rans2[i] + boundL1;
+			p2 = rans4[i] + boundL2;
+			while (p1 >= rans1[i] + boundL1) {
+				temp1 = p1;
+				temp2 = p2;
+				p1--;
+				p2--;
+				if (pos1[p1] == negBuffer)
+					pos1[p1] = temp2;
+				if (pos2[p2] == negBuffer)
+					pos2[p2] = temp1;
+			}
+			
+			// lower score proportionally
+			rans0[i] = s[i]*((double)(rans2[i] - rans1[i] + 1)/(double)(e1[i] - s1[i] + 1));
+		}
+	}
 	free(pos1);
 	free(pos2);
 	
